//
//  CHNYCPlugIn.m
//  CHNYCPlugInDemo
//
//  Created by Jonathan Nathan on 10/11/11.
//  Copyright 2011 JNSoftware LLC. All rights reserved.
//

#import "CHNYCPlugIn.h"

static NSMutableArray *loadedPlugIns = nil;
static NSString *kPlugInPathExtension = @"chnyc-plugin";

@implementation CHNYCPlugIn

@dynamic identifier, displayName, icon;

+ (void)initialize {
	static BOOL isInitialed = NO;
	if (!isInitialed) {
		loadedPlugIns = [[NSMutableArray alloc] init];
		isInitialed = YES;
	}
}

+ (void)loadAllPlugInsAtPath:(NSString *)path {
#if DEBUG
	NSLog(@"CHNYCPlugIn loadAllPlugInsAtPath: %@", path);
#endif
	for (NSString *file in [[NSFileManager defaultManager] contentsOfDirectoryAtPath:path error:nil]) {
		NSString *filePath = [path stringByAppendingPathComponent:file];
		@try {
			if (![[filePath pathExtension] isEqualToString:kPlugInPathExtension]) {
				NSLog(@"CHNYCPlugIn Load Error [1]: %@ is not a plugin.", filePath);
				continue;
			}
			NSBundle *bundle = [NSBundle bundleWithPath:filePath];
			if (!bundle) {
				NSLog(@"CHNYCPlugIn Load Error [2]: %@ is not a plugin bundle.", filePath);
				continue;
			}
			NSError *loadError = nil;
			[bundle loadAndReturnError:&loadError];
			if (loadError) {
				NSLog(@"CHNYCPlugIn Load Error [3]: %@. %@ will not load.", [loadError localizedDescription], filePath);
				continue;
			}
			Class principalClass = [bundle principalClass];
			if (![principalClass isSubclassOfClass:[CHNYCPlugIn class]]) {
				NSLog(@"CHNYCPlugIn Load Error [4]: %@ is not a valid plugin.", filePath);
				continue;
			}
			if (![principalClass respondsToSelector:@selector(defaultPlugIn)]) {
				NSLog(@"CHNYCPlugIn Load Error [5]: %@ is not a valid plugin.", filePath);
				continue;
			}
			id defaultPlugIn = [principalClass defaultPlugIn];
			if (!defaultPlugIn) {
				NSLog(@"CHNYCPlugIn Load Error [6]: plugin %@ did not load correctly.", filePath);
				continue;
			}
			NSString *identifier = [defaultPlugIn identifier];
			if ([[loadedPlugIns valueForKey:@"identifier"] containsObject:identifier]) {
				NSLog(@"CHNYCPlugIn Load Error [7]: duplicate identifier found [%@], %@ will not load.", identifier, filePath);
				continue;
			}
			[loadedPlugIns addObject:defaultPlugIn];
		}
		@catch (NSException *e) {
			NSLog(@"CHNYCPlugIn Load Error: %@ [%@]", [e reason], filePath);
		}
	}
}

+ (void)loadPlugIns {
	NSString *appName = [[NSBundle mainBundle] objectForInfoDictionaryKey:(NSString *)kCFBundleNameKey];
	NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSAllDomainsMask, YES);
	NSUInteger n;
	for (n = 0; n < [searchPaths count]; n++) {
		NSString *path = [[[searchPaths objectAtIndex:n] stringByAppendingPathComponent:appName] stringByAppendingPathComponent:@"PlugIns"];
		[self loadAllPlugInsAtPath:path];
	}
	NSString *path = [[NSBundle mainBundle] builtInPlugInsPath]; 
	if (path) [self loadAllPlugInsAtPath:path];
}

+ (NSArray *)loadedPlugIns {
	return [NSArray arrayWithArray:loadedPlugIns];
}

+ (id)defaultPlugIn {
	[self doesNotRecognizeSelector:_cmd];
	return nil;
}

- (id)init {
	if (self = [super init]) {
	}
	return self;
}

- (void)dealloc {
	[super dealloc];
}

- (NSString *)identifier {
	return [[NSBundle bundleForClass:[self class]] bundleIdentifier];
}

- (NSString *)displayName {
	return NSLocalizedStringFromTableInBundle(@"displayName", @"localized", [NSBundle bundleForClass:[self class]], @"");
}

- (NSImage *)icon {
	NSBundle *bundle = [NSBundle bundleForClass:[self class]];
	NSString *iconPath = [bundle pathForResource:@"flag" ofType:@"png"];
	return (iconPath) ? [[[NSImage alloc] initWithContentsOfFile:iconPath] autorelease] : nil;
}

- (void)sayHello {
	NSString *hello = NSLocalizedStringFromTableInBundle(@"hello", @"localized", [NSBundle bundleForClass:[self class]], @"");
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert setMessageText:hello];
	[alert setInformativeText:@"This is just a silly example of making an app extensible with plugins."];
	NSImage *_icon = [self icon];
	NSImage *alertIcon = [[[NSImage alloc] initWithSize:NSMakeSize(64.0, 64.0)] autorelease];
	[alertIcon lockFocus];
	[_icon drawAtPoint:NSMakePoint(((64.0 - _icon.size.width) / 2.0), (64.0 - _icon.size.height)) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];
	[alertIcon unlockFocus];
	[alert setIcon:alertIcon];
	[alert addButtonWithTitle:@"OK"];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[NSApp activateIgnoringOtherApps:YES];
	NSTimer *timer = [NSTimer timerWithTimeInterval:5.0 target:self selector: @selector(closeAlert:) userInfo:[NSDictionary dictionaryWithObjectsAndKeys:alert, @"alert", nil] repeats:NO];
	[[NSRunLoop currentRunLoop] addTimer:timer forMode:NSModalPanelRunLoopMode];
	[alert runModal];
	[timer invalidate];
}

- (void)closeAlert:(id)timer {
	NSAlert *alert = [[timer userInfo] objectForKey:@"alert"];
	[NSApp endSheet:[alert window] returnCode:NSAlertFirstButtonReturn];
	[[alert window] orderOut:self];
}

@end
