//
//  CHNYCPlugIn.m
//  CHNYCPlugInDemo
//
//  Created by Jonathan Nathan on 10/11/11.
//  Copyright 2011 JNSoftware LLC. All rights reserved.
//

#import "CHNYCPlugIn.h"

static NSMutableDictionary *allPlugIns = nil;
static NSMutableArray *loadedPlugIns = nil;
static NSString *kPlugInPathExtension = @"chnyc-plugin";

@implementation CHNYCPlugIn

@dynamic identifier, displayName, icon;

+ (void)initialize {
	static BOOL isInitialed = NO;
	if (!isInitialed) {
		allPlugIns = [[NSMutableDictionary alloc] init];
		loadedPlugIns = [[NSMutableArray alloc] init];
		isInitialed = YES;
	}
}

+ (void)preloadAllPlugInsAtPath:(NSString *)path {
#if DEBUG
	NSLog(@"CHNYCPlugIn preloadAllPlugInsAtPath: %@", path);
#endif
	for (NSString *file in [[NSFileManager defaultManager] contentsOfDirectoryAtPath:path error:nil]) {
		NSString *filePath = [path stringByAppendingPathComponent:file];
		@try {
			if (![[filePath pathExtension] isEqualToString:kPlugInPathExtension]) {
				NSLog(@"CHNYCPlugIn Preload Error [1]: %@ is not a plugin.", filePath);
				continue;
			}
			NSBundle *bundle = [NSBundle bundleWithPath:filePath];
			if (!bundle) {
				NSLog(@"CHNYCPlugIn Preload Error [2]: %@ is not a plugin bundle.", filePath);
				continue;
			}
			NSDictionary *infoDictionary = [bundle infoDictionary];
			if (!infoDictionary) {
				NSLog(@"CHNYCPlugIn Preload Error [3]: %@ did not return info dictionary.", filePath);
				continue;
			}
			NSString *identifier = [infoDictionary objectForKey:(NSString *)kCFBundleIdentifierKey];
			NSString *version = [infoDictionary objectForKey:(NSString *)kCFBundleVersionKey];
			NSString *name = [infoDictionary objectForKey:(NSString *)kCFBundleNameKey];
			NSDictionary *plugInInfo = [allPlugIns objectForKey:identifier];
			BOOL updatePlugInVersion = (plugInInfo) ? [version isGreaterThan:[plugInInfo objectForKey:@"version"]] : YES;
			if (updatePlugInVersion) [allPlugIns setObject:[NSDictionary dictionaryWithObjectsAndKeys:version, @"version", name, @"name", filePath, @"filePath", nil] forKey:identifier];
		}
		@catch (NSException *e) {
			NSLog(@"CHNYCPlugIn Preload Error: %@ [%@]", [e reason], filePath);
		}
	}
}

+ (void)preloadPlugIns {
	NSString *appName = [[NSBundle mainBundle] objectForInfoDictionaryKey:(NSString *)kCFBundleNameKey];
	NSArray *searchPaths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSAllDomainsMask, YES);
	NSUInteger n;
	for (n = 0; n < [searchPaths count]; n++) {
		NSString *path = [[[searchPaths objectAtIndex:n] stringByAppendingPathComponent:appName] stringByAppendingPathComponent:@"PlugIns"];
		[self preloadAllPlugInsAtPath:path];
	}
	NSString *path = [[NSBundle mainBundle] builtInPlugInsPath];
	if (path) [self preloadAllPlugInsAtPath:path];
}

+ (BOOL)loadPlugIn:(NSString *)plugInIdentifier {
	if ([[loadedPlugIns valueForKey:@"identifier"] containsObject:plugInIdentifier]) return YES; //already loaded
	NSDictionary *plugInInfo = [allPlugIns objectForKey:plugInIdentifier];
	if (!plugInInfo) {
		NSLog(@"CHNYCPlugIn Load Error [1]: %@ is not a known plugin identifier.", plugInIdentifier);
		return NO;
	}
	NSString *filePath = [plugInInfo objectForKey:@"filePath"];
	@try {
		NSBundle *bundle = [NSBundle bundleWithPath:filePath];
		NSError *loadError = nil;
		[bundle loadAndReturnError:&loadError];
		if (loadError) {
			NSLog(@"CHNYCPlugIn Load Error [2]: %@. %@ will not load.", [loadError localizedDescription], filePath);
			return NO;
		}
		Class principalClass = [bundle principalClass];
		if (![principalClass isSubclassOfClass:[CHNYCPlugIn class]]) {
			NSLog(@"CHNYCPlugIn Load Error [3]: %@ is not a valid plugin.", filePath);
			return NO;
		}
		if (![principalClass respondsToSelector:@selector(defaultPlugIn)]) {
			NSLog(@"CHNYCPlugIn Load Error [4]: %@ is not a valid plugin.", filePath);
			return NO;
		}
		id defaultPlugIn = [principalClass defaultPlugIn];
		if (!defaultPlugIn) {
			NSLog(@"CHNYCPlugIn Load Error [5]: plugin %@ did not load correctly.", filePath);
			return NO;
		}
		NSString *identifier = [defaultPlugIn identifier];
		if ([[loadedPlugIns valueForKey:@"identifier"] containsObject:identifier]) {
			NSLog(@"CHNYCPlugIn Load Error [6]: duplicate identifier found [%@], %@ will not load.", identifier, filePath);
			return NO;
		}
		[loadedPlugIns addObject:defaultPlugIn];
	}
	@catch (NSException *e) {
		NSLog(@"CHNYCPlugIn Load Error: %@ [%@]", [e reason], filePath);
		return NO;
	}
	return YES;
}

+ (NSDictionary *)allPlugIns {
	return [NSDictionary dictionaryWithDictionary:allPlugIns];
}

+ (NSArray *)loadedPlugIns {
	return [NSArray arrayWithArray:loadedPlugIns];
}

+ (id)defaultPlugIn {
	[self doesNotRecognizeSelector:_cmd];
	return nil;
}

- (id)init {
	if (self = [super init]) {
	}
	return self;
}

- (void)dealloc {
	[super dealloc];
}

- (NSString *)identifier {
	return [[NSBundle bundleForClass:[self class]] bundleIdentifier];
}

- (NSString *)displayName {
	return NSLocalizedStringFromTableInBundle(@"displayName", @"localized", [NSBundle bundleForClass:[self class]], @"");
}

- (NSImage *)icon {
	NSBundle *bundle = [NSBundle bundleForClass:[self class]];
	NSString *iconPath = [bundle pathForResource:@"flag" ofType:@"png"];
	return (iconPath) ? [[[NSImage alloc] initWithContentsOfFile:iconPath] autorelease] : nil;
}

- (void)sayHello {
	NSString *hello = NSLocalizedStringFromTableInBundle(@"hello", @"localized", [NSBundle bundleForClass:[self class]], @"");
	NSAlert *alert = [[[NSAlert alloc] init] autorelease];
	[alert setMessageText:hello];
	[alert setInformativeText:@"This is just a silly example of making an app extensible with plugins."];
	NSImage *_icon = [self icon];
	NSImage *alertIcon = [[[NSImage alloc] initWithSize:NSMakeSize(64.0, 64.0)] autorelease];
	[alertIcon lockFocus];
	[_icon drawAtPoint:NSMakePoint(((64.0 - _icon.size.width) / 2.0), (64.0 - _icon.size.height)) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0];
	[alertIcon unlockFocus];
	[alert setIcon:alertIcon];
	[alert addButtonWithTitle:@"OK"];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[NSApp activateIgnoringOtherApps:YES];
	NSTimer *timer = [NSTimer timerWithTimeInterval:5.0 target:self selector: @selector(closeAlert:) userInfo:[NSDictionary dictionaryWithObjectsAndKeys:alert, @"alert", nil] repeats:NO];
	[[NSRunLoop currentRunLoop] addTimer:timer forMode:NSModalPanelRunLoopMode];
	[alert runModal];
	[timer invalidate];
}

- (void)closeAlert:(id)timer {
	NSAlert *alert = [[timer userInfo] objectForKey:@"alert"];
	[NSApp endSheet:[alert window] returnCode:NSAlertFirstButtonReturn];
	[[alert window] orderOut:self];
}

- (NSString *)sdefPath {
	NSBundle *bundle = [NSBundle bundleForClass:[self class]];
	NSString *sdefPath = [bundle pathForResource:@"plugin" ofType:@"sdef"];
	return (sdefPath) ? sdefPath : nil;
}

+ (NSString *)translateText:(NSString *)inputString toLanguage:(NSString *)language {
	return [NSString stringWithFormat:@"%@ in %@", inputString, language];
}

@end
